/*
 *  Dimitri KRAVTCHUK (c) 1995-2016
 *  dimitri (dot) kravtchuk (at) free (dot) fr
 *
 *  ...   : 1997... - long dev progress ;-)
 *  v.2.5 : 2015.12 - aligned with STAT-service v.5.0
 *  v.2.6 : 2016.01 - various code remastering..
 */

#include <stdio.h>
#include <signal.h>
#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <time.h>
#include <sys/time.h>           /* struct timeval */
#include <unistd.h>
#include <stdlib.h>
#include <string.h>


#define VERSION           "(dim) v.2.6"

struct sockaddr_in sinme;
struct sockaddr_in sinhim;
struct sockaddr_in frominet;

extern int errno;

static int fd;                         /* fd of network socket */
short BreakAlarm= 0;                   /* Alarm BreakConnection */

void err( char *s, int sst );
void msg( char *s );
int Nread();
int Nwrite();

void sigpipe() {}

void sigalrm()
{
 signal( SIGALRM, sigalrm );
 if( BreakAlarm ) close( fd );
}


void Usage( char *cmd )
{
 fprintf( stderr,
 "%s %s \n"
 "Usage: STATcmd [-k] [-t numb] [-b nsec] -h host [-p port] -c command \n"
 "  -b nsec   -- break connection after nsec-seconds of inactivity\n"
 "  -t numb   -- try to connect only numb times (default: 0 (unlimited)) \n"
 "  -p port   -- STAT-service port number (default: 5000)\n"
 "  -k        -- keep alive\n",
  cmd, VERSION
 );

 exit( -2 );
}


int main( int argc, char *argv[] )
{
 int domain, fromlen;

 short KeepAlive= 0;
 short port= 5000;               /* TCP port number */
 char *cmd= NULL;                /* STAT command */
 char *host= NULL;               /* ptr to name of host */
 struct hostent *addr;
 unsigned long addr_tmp;
 int i, cnt, sst, nb;
 char buff[1024];
 long T0, T1;
 int Try= 0;
 char *NO_CNN= "STAT *** NO CONNECTION\n";

 if( argc < 3 )  Usage( argv[0] );

 for( i= 1; i < argc; i++ )
 {
  if( strcmp( argv[i], "-k" ) == 0 )  { KeepAlive= 1;                   continue; }
  if( strcmp( argv[i], "-t" ) == 0 )  { Try= atoi( argv[++i] );         continue;  }
  if( strcmp( argv[i], "-p" ) == 0 )  { port= atoi( argv[++i] );        continue;  }
  if( strcmp( argv[i], "-b" ) == 0 )  { BreakAlarm= atoi( argv[++i] );  continue;  }
  if( strcmp( argv[i], "-h" ) == 0 )  { host= argv[++i];                continue;  }
  if( strcmp( argv[i], "-c" ) == 0 )  { cmd= argv[++i];                 continue;  }

  Usage( argv[0] );
 }

 if( BreakAlarm < 0 ) BreakAlarm= 0;
 if( host == NULL || cmd == NULL || strlen(cmd) == 0 )  Usage( argv[0] );
 bzero((char *)&sinhim, sizeof(sinhim));

 if (atoi(host) > 0 )
 {
  /* Numeric */
  sinhim.sin_family = AF_INET;
  sinhim.sin_addr.s_addr = inet_addr( host );
 }
 else
 {
  if( (addr= gethostbyname( host )) == NULL )   err( "bad hostname", -1 );
  sinhim.sin_family = addr->h_addrtype;
  bcopy( addr->h_addr, (char*)&addr_tmp, addr->h_length );
  sinhim.sin_addr.s_addr= addr_tmp;
 }

 sinhim.sin_port = htons(port);
 sinme.sin_port = 0;             /* free choice */
 sinme.sin_family = AF_INET;

 time( &T0 );

 for( nb= 0; ; )
 {
  signal( SIGPIPE, sigpipe );
  signal( SIGALRM, sigalrm );
  if( BreakAlarm )  alarm( BreakAlarm );
  if( (fd= socket( AF_INET, SOCK_STREAM, 0)) < 0 )    err( "socket", fd );
  if( (sst= bind( fd, (struct sockaddr *)&sinme, sizeof(sinme) )) < 0 )   err( "bind", sst );

  if( connect( fd, (struct sockaddr *)&sinhim, sizeof(sinhim) ) < 0 )
  {
   sst= write( 1, NO_CNN, strlen( NO_CNN ) );
   close( fd );
   if( sst <= 0 )  exit(0);  /** Broken pipe... **/

   if( Try && ++nb >= Try )  exit(0);
   sleep( 20 );
   continue;
  }

  nb= 0;
  time( &T1 );

  sprintf( buff, "STAT *** OK CONNECTION %ld sec.\n", T1-T0 );
  write( 1, buff, strlen( buff ) );

  if( (sst= Nwrite( fd, cmd, strlen(cmd) )) <= 0 )  err( "Nwrite", sst );
  if( (cnt= Nread( fd, buff, sizeof(buff) )) <= 0 )  err( "Nread", cnt );

  write( 1, buff, strlen( buff ) );
  if( strncmp( buff, "STAT *** BAD COMMAND", 20 ) == 0 )   break;

  while( 1 )
  {
   time( &T0 );   /* timestamp to get inactivity info */

   if( (cnt= Nread( fd, buff, sizeof(buff) )) <= 0 )
   {
    if( !KeepAlive )  { close( fd ); exit( 0 ); }

    sst= write( 1, NO_CNN, strlen( NO_CNN ) );
    close (fd );
    if( sst <= 0 )  exit(0);  /* Broken pipe... */

    sleep( 20 );
    break;
   }

   if( write( 1, buff, strlen( buff ) ) <= 0 )  /* Broken pipe? */
   {
    close( fd );
    exit( 0 );
   }
  }
 }

 close( fd );
 exit( 0 );
}


void err( char *s, int sst )
{
 fprintf( stderr, "## client pid-%d: %s sst: %d errno: %d\n", getpid(), s, sst, errno );
 perror( s );
 exit( -2 );
}


void msg( char *s)
{
 fprintf( stderr, "## client pid-%d: %s\n", getpid(), s );
}


int Nread( int fd, char *buf, int count )
{
 int cnt;

 if( BreakAlarm )  alarm( BreakAlarm );
 cnt= read( fd, buf, count );
 if( cnt >= 0 ) buf[cnt]= '\0';

 return( cnt );
}


int Nwrite( int fd, char *buf, int count )
{
 return( write( fd, buf, count ) );
}

